/*
 * @copyright		© Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/content/inc/megamenu
 *
 * @revision		$Revision: 1.11 $
 */

define("de_epages/content/inc/megamenu", [
	"jquery",
	"ep",

	"jquery/masonry"
], function ($, ep) {

	return {
		init: function(options){
			//define defaults for mega menu
			var defaultValues = {
					letterCount : 20,
					fallbackColumWidth : 200,
					minOffsetLeft: 10,
					menuDelay: 500,
					isEditorMode: false
				},
				options = $.extend(defaultValues,options),
				menu = $(options.selector),
				mainContainer = menu.parents('.PropertyContainer'),
				mainWidth = mainContainer.width(),
				mainLeftPadding = parseInt(mainContainer.css('padding-left')),
				activeMenu,
				activeListEntry,
				correctContainer;

			if(menu){
				//create element to test current letter width
				var letterWidthNode = $('<span>').appendTo('body');

				//use variety of widest letters to calculate max letter width
				letterWidthNode
					.text('PMWXGOQ')
					.addClass('ep-megamenu-second-level-font-check');

				var letterWidth = Math.floor(letterWidthNode.width() / 7);

				letterWidthNode.remove();

				var columWidth = (letterWidth ? letterWidth * options.letterCount : options.fallbackColumWidth);

				menu.find('.ep-megamenu-second-level-element').css({
					width: columWidth  + 'px'
				});

				var items = menu.children('.ep-megamenu-first-level-element');

				//menu hover
				menu.on('mouseenter','.ep-megamenu-first-level-element',function(evt){
					var item = $(this),
						container = item.children('.ep-megamenu-container');
					if(container.length){
						var contentList = container.children('.ep-megamenu-second-level-group');
						container.clearQueue().queue(function(){
							//hide last menu container
							if(activeListEntry){
								activeListEntry.removeClass('active-item');
							}
							activeListEntry = item;
							activeListEntry.addClass('active-item');
							if(activeMenu){
								activeMenu.clearQueue().hide();
							}

							activeMenu = container;
							container.show();
							contentList.masonry();
							//check li height
							contentList.children('.ep-megamenu-second-level-element').each(function(){
								var entry = $(this),
									entryOuterWidth = entry.outerWidth(true),
									entryHeight = entry.outerHeight(true),
									entryTopOffset = entry.position().top,
									visibleEntryCount = entry.find('.ep-megamenu-third-level-element:visible').length,
									containerHeight = container.height(),
									containerMaxHeight = parseInt(container.css('max-height'),10);
								if(entryOuterWidth !== columWidth){
									entry.css({
										width: columWidth - (entryOuterWidth - columWidth) + 'px'
									});
								}
								containerHeight = containerHeight < containerMaxHeight ? containerMaxHeight : containerHeight;
								//hide elements until list fits into given height
								contentList.masonry();
								for(var i = 0; i < visibleEntryCount; i++){
									entryTopOffset = entry.position().top;
									if(entryHeight + entryTopOffset > containerHeight){
										entry.find('.ep-megamenu-more').show();
										entry.find('.ep-megamenu-third-level-element:visible').last().hide();
										entryHeight = entry.outerHeight(true);
									}
								}
							});
							var contentListHeight = contentList.height();
							if(contentListHeight > parseInt(container.css('max-height'),10)){
								container.css({
									'max-height': contentListHeight + 'px'
								});
								contentList.masonry();
							}
						});

						//correct container position
						if(correctContainer){
							correctContainer(container);
						}
					}
				});

				//hover out
				menu.on('mouseleave','.ep-megamenu-first-level-element',function(evt){
					var item = $(this),
						container = item.children('.ep-megamenu-container');
					container.clearQueue().delay(options.menuDelay).queue(function(){
						container.hide();
						if(activeListEntry){
							activeListEntry.removeClass('active-item');
						}
					});
				});

				items.each(function(){
					var item = $(this),
						container = item.children('.ep-megamenu-container'),
						activeMarker = item.find('.active-marker'),
						itemAnchor = item.children('a');

					activeMarker.css({
						'left': ((itemAnchor.width() / 2) - 5) + 'px'
					});

					if(container.length && mainContainer.length){
						correctContainer = function(container){
							//calculate first width of menu container will be updated at hover event but is needed now for initalization
							var contentList = container.children('.ep-megamenu-second-level-group'),
								countSecondLevel = container.find('.ep-megamenu-second-level-element').length,
								//mainWidth = container.parents('.PropertyContainer').width(),
								maxColumnCount =  Math.floor(mainWidth / columWidth),
								currentColumWidth = countSecondLevel < maxColumnCount ? countSecondLevel * columWidth : (maxColumnCount * columWidth),
								endPosition = (container.parent('.ep-megamenu-first-level-element').offset().left - container.parents('.PropertyContainer').offset().left) + currentColumWidth;

							container.css({
								width: (currentColumWidth) + 'px'
							});

							//set left after width was set to read outerWidth of container
							//move container 10px left for design issues
							var containerOuterWidth = container.outerWidth(true),
								offsetLeft = mainWidth - containerOuterWidth;

							if(offsetLeft < options.minOffsetLeft){
								offsetLeft = options.minOffsetLeft;
							}

							//check editormode
							var left = 0,
								offsetParentNode = item.offsetParent();

							//if offsetParent is a PropertyContainer then we are in the SF
							if(!options.isEditorMode){
								left = offsetLeft + mainLeftPadding;
							}
							//if not we are in the MBO and we need to do a little bit of calculation to get the right offset left value
							else{
								var mainEndPoint = mainWidth + mainContainer.offset().left,
									containerEndPoint = offsetParentNode.offset().left + containerOuterWidth;
								left = mainEndPoint - containerEndPoint;
							}

							container.css({
								left: endPosition > mainWidth ? (left) + 'px' : ''
							});
						};

						correctContainer(container);
					}
				});

				//initalize masonry
				menu.find('.ep-megamenu-second-level-group')
					.masonry({
						itemSelector : '.ep-megamenu-second-level-element',
						columnWidth: columWidth
					});
			}
			//add ActiveClass to current selected main node
			$('.Category-' + ep.config.objectId + '.NavigationElement').addClass('Active');
		}
	};
});