define(['testsuite/qunit', 'jquery', 'de_epages/customform/ui/customform', '$ready!'], function (QUnit, $, de_epages) {

	QUnit.module("de_epages/customform/ui/customform");

	QUnit.asyncTest("de_epages/customform/ui/customform", function() {

		var objSend = {
			   "Alias" : "RadioButtonFormField",
			   "IsMandatory" : 0,
			   "Options" : [
			      {
			         "Alias" : "Option509907DAAC14041D60A1",
			         "OptionName" : "Option 1"
			      },
			      {
			         "Alias" : "Option509907DAAC14041D6044",
			         "OptionName" : "Option 2"
			      },
			      {
			         "Alias" : "Option50990E70AC14041D6065",
			         "OptionName" : "Option 3"
			      }
			   ],
			   "HasOptions" : 1,
			   "FieldName" : "Optionen",
			   "AdditionalHelpText" : "Hier sind Optionen"
			},

			objResult = {
		   "Alias" : "RadioButtonFormField",
		   "IsMandatory" : 0,
		   "Options" : {
		      "16762" : [
		         {
		            "OptionDate" : "2012-11-06T13:51:38",
		            "OptionName" : "Option 1",
		            "OptionLangID" : "1"
		         },
		         {
		            "OptionDate" : "2012-11-06T13:51:38",
		            "OptionName" : "o1",
		            "OptionLangID" : "2"
		         }
		      ],
		      "16769" : [
		         {
		            "OptionDate" : "2012-11-06T14:19:44",
		            "OptionName" : "Option 3",
		            "OptionLangID" : "1"
		         },
		         {
		            "OptionDate" : "2012-11-06T14:19:44",
		            "OptionName" : "o3",
		            "OptionLangID" : "2"
		         }
		      ],
		      "16763" : [
		         {
		            "OptionDate" : "2012-11-06T13:51:38",
		            "OptionName" : "Option 2",
		            "OptionLangID" : "1"
		         },
		         {
		            "OptionDate" : "2012-11-06T13:51:38",
		            "OptionName" : "o2",
		            "OptionLangID" : "2"
		         }
		      ]
		   },
		   "OptionsOrder" : [
		      "16762",
		      "16763",
		      "16769"
		   ],
		   "NameOrAlias" : {
		      "1" : "Optionen",
		      "2" : "RadioButtonFormField"
		   },
		   "HasOptions" : 1,
		   "LocalizedTexts" : [
		      {
		         "LanguageID" : "1",
		         "FieldName" : "Optionen",
		         "AdditionalHelpText" : "Hier sind Optionen"
		      },
		      {
		         "LanguageID" : "2",
		         "FieldName" : "",
		         "AdditionalHelpText" : ""
		      }
		   ]
		};

		/* START - fake ajax request */
		$.ajax = function( o ){
			var result = $.Deferred(),
				resolveSend;

			if(o.data && o.data.length){
				$.each(o.data, function(key, current){
					if((current.name == 'ChangeAction') && (current.value == 'JSONAlterField')){
						resolveSend = true;
					}
				});
			}

			if( o.data && o.data.ViewAction == 'JSONGetFieldData' ){
				result.resolve(objResult);
			}else if(resolveSend){
				result.resolve(objSend);
			}else{
				result.reject();
			}

			return result;
		}
		/* END - fake ajax request */

		var customform = de_epages('.editCustomField').customformUiCustomform({
			width: 800,
			height: 400,
			languageId: '1',
			languages: {'2' : 'English', '1' : 'Deutsch'}
		});

		// prototype methods defined
		var fnCheck = true;

		$.each( ['_create', '_handleNewOption', '_getFieldData', '_processFieldData'], function( i, fn ){
			fnCheck = !$.isFunction( $.ui.customformUiCustomform.prototype[fn] ) ? false : fnCheck;

			QUnit.ok($.isFunction($.ui.customformUiCustomform.prototype[fn])===true, 'method' + fn + ' is defined');
		});

		QUnit.equal(
			fnCheck,
			true,
			"each => de_epages.customform.ui.customform.prototype => $.isFunction()"
		);

		customform.trigger('click');

		setTimeout(function(){
			var customformWidget = customform.customformUiCustomform().data(),
				customformInstance = customformWidget.uiCustomformUiCustomform,
				customDialog = customformInstance.dialog,
				optionsHolder = customDialog.find('.epCustomFormOptions');

			// check if dialog is visible/open
			QUnit.ok(customform.is(':visible')===true, 'dialog is visible');

			// check if value for textfield was loaded
			QUnit.ok($('#rewriteFieldNameId').val() != "", '"' + $('#rewriteFieldNameId').val() + '" as value for text field name was set');

			// check number of options before deleting the first one
			QUnit.ok(optionsHolder.find('li:visible').length == 3, 'number of options before deleting the first one: ' + optionsHolder.find('li:visible').length);

			// check number of FieldName inputs (have to be as much as number of languages)
			QUnit.equal(
				customDialog.find('input[name=FieldName]').length,
				2,
				"check number of FieldName inputs"
			);

			// check number of AdditionalHelpText inputs (have to be as much as number of languages)
			QUnit.equal(
				customDialog.find('input[name=AdditionalHelpText]').length,
				2,
				"check number of AdditionalHelpText inputs"
			);

			customformInstance.dialog.find('.epCustomFormOptions a[data-deleteoptionid]:first').trigger('click');

			setTimeout(function(){

				QUnit.ok(optionsHolder.find('li:visible').length == 2, 'number of options after deleting the first one: ' + optionsHolder.find('li:visible').length);

				// create a new option
				customDialog.find('#addNewOptionLink').trigger('click');

				// check number of options after adding a new one
				QUnit.ok(optionsHolder.find('li:visible').length == 3, 'number of options after adding a new one: ' + optionsHolder.find('li:visible').length);
				// delet new option again
				optionsHolder.find('a.deleteNewOptionLink').trigger('click');

				setTimeout(function(){

					QUnit.ok(optionsHolder.find('li:visible').length == 2, 'number of options after deleting the last added one: ' + optionsHolder.find('li:visible').length);


					// trigger click to send form and close dialog
					$('button[type=submit]').trigger('click');

					QUnit.start();

				}, 1200);

			}, 1200);

		}, 300);
	});
});