#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::TranslationOfDHLServices
# state        private
#----------------------------------------------------------------------------------------
# description  translation for the service names ( they will translated from german to englisch and back)
#========================================================================================
package TestTranslationOfDHLService;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::TranslationOfDHLServices;
use Test::More;

sub Main {
    my $TranslationOfDHLServices = DE_EPAGES::DHLIntraship::API::TranslationOfDHLServices->new(
        {
            'Mglichlkeit1' => 'Option1',
            'Mglichlkeit2' => 'Option2',
        },
        {'Gruppe' => 'Group'},
    );
    GermanToEnglisch( $TranslationOfDHLServices );
    EnglischToGerman( $TranslationOfDHLServices );
    done_testing();
}

sub GermanToEnglisch {
    my $TestName = (caller(0))[3];
    my ($TranslationOfDHLServices) = @_;
    my $aService = $TranslationOfDHLServices->toEnglish( [
        {
            'Name' => 'Gruppe',
            'SelectedOption' => 'Mglichlkeit2',
            'Options' => [
                'Mglichlkeit1',
                'Mglichlkeit2',
            ],
        },
        {
            'Name' => 'NoTransGruppe',
            'SelectedOption' => 'NoTransMglichlkeit2',
            'Options' => [
                'NoTransMglichlkeit1',
                'NoTransMglichlkeit2',
            ],
        }
         ] );
    #test translated
    is($aService->[0]->{'Name'},'Group',"$TestName - Translation of Groupname - Group");
    is($aService->[0]->{'SelectedOption'},'Option2',"$TestName - Translation of SelectedOption - Option2");
    is_deeply(
        $aService->[0]->{'Options'},
        ['Option1', 'Option2'],
         "$TestName - Translation of Options - Array with Option names");
    # test untranslated
    is($aService->[1]->{'Name'},'NoTransGruppe',"$TestName - no Translation of Groupname - Group");
    is($aService->[1]->{'SelectedOption'},'NoTransMglichlkeit2',"$TestName - no Translation of SelectedOption - NoTransMglichlkeit2");
    is_deeply(
        $aService->[1]->{'Options'},
        ['NoTransMglichlkeit1', 'NoTransMglichlkeit2'],
         "$TestName - no Translation of Options - Array with Option names");
}

sub EnglischToGerman {
    my $TestName = (caller(0))[3];
    my ($TranslationOfDHLServices) = @_;
    my $aService = $TranslationOfDHLServices->toGerman( [{
        'Name' => 'Group',
        'SelectedOption' => 'Option2',
        'Options' => [
            'Option1',
            'Option2',
        ],
    }] );
    is($aService->[0]->{'Name'},'Gruppe',"$TestName - Translation of Groupname - Group");
    is($aService->[0]->{'SelectedOption'},'Mglichlkeit2',"$TestName - Translation of SelectedOption - Option2");
    is_deeply(
        $aService->[0]->{'Options'},
        ['Mglichlkeit1', 'Mglichlkeit2'],
         "$TestName - Translation of Options - Array with Option names");
}

RunScript( Sub => \&Main );