#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::service
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for service validation
#========================================================================================
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::Services;
use Test::Exception;
use Test::More;

sub Main {
    RegressionTestServices();
    done_testing();
}

sub RegressionTestServices{
    my $ServicesObject = DE_EPAGES::DHLIntraship::API::Services->new();

    my $InjectClass = MockObjectPM();

    my $aServieList = [ {'ServiceName'=>'ServiceName1', 'ServiceGroup'=>'ServiceGroup1', 'ServiceAmount'=>'ServiceAmount1'}, ];
    $ServicesObject->save( $InjectClass, $aServieList);
    TestInsertChildParametersAsGroup( $InjectClass, $aServieList);

    $aServieList = [ {'ServiceName' => 'ServiceName2', 'ServiceGroup' => 'ServiceGroup2'}, ];
    $ServicesObject->save( $InjectClass, $aServieList);
    TestInsertChildParametersAsGroup( $InjectClass, $aServieList);

    $aServieList = [['eins','zwei']];
    throws_ok(sub{$ServicesObject->save( $InjectClass, $aServieList)},qr/WrongInputType/,'RegressionTestServices - checks the Error case for wrong inputtype - WrongInputType');

    $aServieList = [ {'ServiceName' => 'ServiceName2', 'ServiceGroup' => undef}, ];
    is($ServicesObject->save( $InjectClass, $aServieList),undef,'RegressionTestServices - if ServiceGroup is undef nothing should return - undef');

    $aServieList = [ {'ServiceName' => undef, 'ServiceGroup' => 'ServiceGroup2'}, ];
    is($ServicesObject->save( $InjectClass, $aServieList),undef,'RegressionTestServices - if ServiceName is undef nothing should return - undef');

    $aServieList = [ {'ServiceName' => undef, 'ServiceGroup' => undef}, ];
    is($ServicesObject->save( $InjectClass, $aServieList),undef,'RegressionTestServices - if ServiceName and ServiceGroup are undef nothing should return - undef');
}

sub TestInsertChildParametersAsGroup{
    my ( $InjectClass, $aInfo ) = @_;
    my $hInfo = $$aInfo[0];
    is($InjectClass->{'Class'},         'DHLIntrashipService', 'result insertChild - check if the first parameter is class name - DHLIntrashipService');
    is($InjectClass->{'Alias'},         $hInfo->{'ServiceName'}, "result insertChild - check if in the $hInfo->{Alias} is right Value - $hInfo->{'ServiceName'}");
    is($InjectClass->{'ServiceGroup'},  $hInfo->{'ServiceGroup'}, "result insertChild - check if in the $hInfo->{Alias} is right Value - $hInfo->{'ServiceGroup'}");
    is($InjectClass->{'ServiceAmount'}, $hInfo->{'ServiceAmount'}, "result insertChild - check if in the $hInfo->{Alias} is right Value - $hInfo->{'ServiceAmount'}");
}

sub MockObjectPM {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages RequireCommentForExportedFunctions RequireArgUnpacking)
        package Object;
        sub deleteChildren { return; };
        sub folder {return $_[0];};
        sub insertChild {
            my $self = shift;
            my ($Class, $hInfo) = @_;
            $self->{'Class'}         = $Class;
            $self->{'Alias'}         = $hInfo->{'Alias'};
            $self->{'ServiceGroup'}  = $hInfo->{'ServiceGroup'};
            $self->{'ServiceAmount'} = $hInfo->{'ServiceAmount'};
        };
        return bless {}, 'Object';
    }
}
RunScript( Sub => \&Main );