use Test::Simple tests => 8;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );

#----------------------------------------------------------------------------------------
# description  Test of _GetCurrencyNotInProfile private function.
#----------------------------------------------------------------------------------------
sub testGetCurrencyNotInProfile {
    my ($aActiveCurrencies, $RemoteCurrencies, $aResult) = @_;

    # remote currencies can be an array, an string or even empty, so we set it as an array
    my $aRemoteCurrencies = ref($RemoteCurrencies) eq 'ARRAY' ? $RemoteCurrencies : [$RemoteCurrencies];

    my %LookupProfile = map { $_ => 'yes' } @$aRemoteCurrencies;
    my @CurrencyNotInProfile = ();

    foreach my $Currency (@$aActiveCurrencies) {
        push( @CurrencyNotInProfile, {'Currency' => $Currency->{'CurrencyID'}} ) unless exists $LookupProfile{$Currency->{'CurrencyID'}};
    }

    ok(@CurrencyNotInProfile == @$aResult, 'array of hashes match');
}



#--------------------------------------------------------------------------------------
RunScript(
    'Sub' => sub {
        my @ActiveCurrencies = (
            {
                'CurrencyID' => 'EUR',
                'Factor' => undef,
                'IsActivated' => 1,
                'IsUsed' => 1,
                'LeadCurrencyID' => undef,
                'MinorUnit' => 2,
                'Name' => 'Euro',
                'NumCode' => 978,
                'Position' => 10,
                'ShopID' => 16402,
                'Symbol' => ''
            },
            {
                'CurrencyID' => 'GBP',
                'Factor' => undef,
                'IsActivated' => 1,
                'IsUsed' => 1,
                'LeadCurrencyID' => undef,
                'MinorUnit' => 2,
                'Name' => 'Pound Sterling',
                'NumCode' => 826,
                'Position' => 20,
                'ShopID' => 16402,
                'Symbol' => ''
            }
        );
        ok( 1, 'test started' );
        # Parameters: 1. array.ref of currencies to check coverage (they correspond with active currencies in the shop)
        #             2. array.ref (many) or string (one) of currencies allowed to merchant by EasyDebit
        #             3. Result. array.ref of currencies not covered by EasyDebit
        testGetCurrencyNotInProfile(\@ActiveCurrencies, ['EUR','GBP'], []);
        testGetCurrencyNotInProfile(\@ActiveCurrencies, ['EUR','USD'], [{'Currency' => 'GBP'}]);
        testGetCurrencyNotInProfile(\@ActiveCurrencies, 'EUR'        , [{'Currency' => 'GBP'}]);
        # corner cases
        testGetCurrencyNotInProfile(\@ActiveCurrencies, ''           , [{'Currency' => 'EUR'},{'Currency' => 'GBP'}]);
        testGetCurrencyNotInProfile(\@ActiveCurrencies, undef        , [{'Currency' => 'EUR'},{'Currency' => 'GBP'}]);
        testGetCurrencyNotInProfile([],                 ''           , []);
        testGetCurrencyNotInProfile([],                 undef        , []);

        return;
    }
);
