/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.slider
 *
 * @revision		$Revision: 1.6 $
 */

define("ep/ui/slider", [
	"jquery",
	"ep",

	"jquery/ui/widget",
	"jquery/ui/slider",
	"ep/ui/tooltip"
], function ($, ep) {

	$.widget('ui.uiSlider', $.ui.slider, {

		options : {
			label :	'',
			unit : '',
			info : null
		},

		_create : function () {
			var self = this,
					o = $.extend(self.options, $.parseJSON(self.element.attr('data-ep-ui-slider')));

			// Create UI elements, add CSS classes and set initial texts.
			self.labelNode = $('<div>')
				.text(o.label)
				.addClass('ep-uiSlider-label');

			if (o.info) {
				self.infoNode = $('<span>').text(o.info.alt);

				$('<img/>')
					.addClass('BubbleHelp')
					.attr({
						'src' : o.info.src,
						'alt' : o.info.alt
					})
					.appendTo(self.labelNode);
			}

			self.valueNode = $('<div>').addClass('ep-uiSlider-value');
			self.sliderNode = $('<div>').addClass('ep-uiSlider-slider');

			self.element
				.addClass('ep-uiSlider-wrap')
				.append(
					self.labelNode,
					self.valueNode,
					self.sliderNode
				);

			// Create (ep.ui) *tooltip*.
			if (o.info) {
				self.infoNode.uiTooltip();
			}

			// Create (jQuery) *slider*.
			self.element = self.sliderNode;

			self._superApply(arguments);

			self.element = self.element.parent();

			// Add event handling.
			self.element.on('slide', function (event, ui) {
				self.valueNode.text(ui.value + (o.unit ? (' ' + o.unit) : ''));
			});

			// Render *valueNode*.
			self.element.trigger('slide', {value : o.value});
		},

		destroy : function () {
			var self = this;

			if (self.infoNode) {
				self.infoNode.remove();
			}

			self.labelNode.remove();

			self.valueNode.remove();

			self.element.removeClass('ep-uiSlider-wrap');

			self._superApply(arguments);

			self.sliderNode.remove();
		}

	});

	return ep;

});