/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.uri
 *
 * @revision		$Revision: 1.7 $
 */

define("ep/uri", [
	"jquery",
	"ep",
	"util/string",

	"jquery/param"
], function ($, ep, str) {

	var uriExpr = /^((?:(.+)\:\/\/([^\/]+))?((?:\/|[^\?#]).*?)?(\?.*?)?(#.*)?)$/,
		absExpr = /^((http|https|ftp)\:\/\/|\/)/g;

	ep.extend({
		uriSegment: function( obj ) {
			var ret;
			if( typeof obj === 'string' ){
				var parts = obj.match(uriExpr);

				ret = parts ? {
					'href': parts[1] || '',
					'protocol':parts[2] || '',
					'host': parts[3] || '',
					'path': parts[4] ? parts[4].replace(/\/?(.*)$/i,"$1") : '',
					'query': parts[5] || '',
					'anchor': parts[6] || '',
					'parameters': parts[5] ? $.param( parts[5] ) : {}
				} : {};
			}
			else if( typeof obj === 'object' ){
				ret = '';
				ret += obj.protocol ? obj.protocol + '://' : '';
				ret += obj.host ? obj.host + '/' : '';
				ret += obj.path ? obj.path.replace(/\/?(.*)$/i,"$1") : '';

				if( obj.parameters ){
					var queryString = $.param(obj.parameters);
					ret += queryString ? '?'+queryString : '';
				}
				else{
					ret += obj.query || '';
				}

				ret += obj.anchor || '';
			}
			return ret;
		},

		uriQuery: function (obj)  {
			ep.deprecated("'ep.uriQuery()' is replaced by 'jQuery.param()'", true);

			return $.param(obj);
		},

		uriIsAbsolute: function( url ){
			return absExpr.test(url);
		},

		uriToAbsolute: function( url, baseUrl ){
			return (baseUrl || ep.config.baseUrl).replace(/(\/)$/,'') + "/" + ep.uriToRelative( url, baseUrl );
		},

		uriToRelative: function( url, baseUrl ){
			var srcUrl = url;
			url = ep.uriSegment( url );
			baseUrl = ep.uriSegment( baseUrl || ep.config.baseUrl );

			if ( (url.protocol && url.protocol !== baseUrl.protocol) || (url.host && url.host !== baseUrl.host) ){
				return srcUrl;
			}

			var parts = str.escExpStr( baseUrl.path.replace(/^\/?(.*)$/i,"$1").replace(/(.*)\/$/i,"$1") ).split('\\/'),
				exprArr = [];

			for( var i=parts.length-1 ; i>=0 ; i-- ){
				exprArr.unshift(parts[i]);

				var expr = new RegExp( '^(' + exprArr.join('\\/') + ')', '' );

				if( expr.test(url.path) ){
					url.path = url.path.replace(expr,'').replace(/^(\/)/,'');
					break;
				}
			}

			return ep.uriSegment({
				'path': url.path,
				'query': url.query,
				'anchor': url.anchor,
				'parameters': url.params
			});
		}
	});

	return ep;

});