/*global define*/

define("de_epages/mediagallery/filemanager/file-view", [
    "backbone",
    "jquery",
    "$tmpl!./file-view",
    "$dict!../dictionary",

    "ep/ui/tooltip"
], function (Backbone, $, template, mediagalleryDict) {
    "use strict";

    /*
     * @dictionary       de_epages.mediagallery.dictionary
     *
     * @translation      {FileAlreadyExists}
     *                   {UnsupportedFileFormat}
     */

    var tFileAlreadyExists = mediagalleryDict.translate("FileAlreadyExists"),
        tUnsupportedFileFormat = mediagalleryDict.translate("UnsupportedFileFormat"),

        view = Backbone.View.extend({
            tagName: "li",
            className: "ep-mediagallery-fileContainer",
            template: template,

            events: {
                "click .ep-mediagallery-itemName": "edit",
                "keyup input": "onKeyup",
                "blur input": "save"
            },

            initialize: function () {
                this.model
                    // render view when an attribute changes
                    .on("change", this.render, this)
                    // display a tooltip when a file with this name already exists
                    .on("fileAlreadyExists", this.fileAlreadyExists, this);
            },

            render: function () {
                var tmpl = this.template({
                    placeholder: this.model.get("placeholder"),
                    name: this.model.get("name"),
                    size: Math.round(this.model.get("size") / 1024) + " kB",
                    measure: (this.model.get("width") !== null) ? this.model.get("width") + " x " + this.model.get("height") : undefined,
                    path: (this.model.get("preview") !== undefined) ? this.model.get("preview") : this.model.get("fullpath")
                });

                $.extend(this, $.tmplItem(tmpl).elements);

                this.$el.html(tmpl.children()).data("itemID", this.model.get("id"));

                if (!this.model.get("selectable")) {
                    this.$el.addClass("disabled");
                    $('<span class="Tooltip">' + tUnsupportedFileFormat + '</span>').uiTooltip({
                        context: this.$el,
                        orientation: "right",
                        offsetAdjust: [-60, 30]
                    });
                }

                return this;
            },


            // edit name of file
            edit: function () {
                if (this.model.get("metadata").type === "MediaGallery") {
                    var extension = this.model.get("extension"),
                        value = this.model.get("name").replace(new RegExp(extension, 'g'), "");

                    this.itemName.replaceWith('<input />');
                    this.input = this.$('input');
                    this.input.val(value).select().after($("<span>").after($("<br/>")).text(extension));
                }
            },

            onKeyup: function (e) {
                if (e.which === 13 || e.which === 27) {
                    this.save(e);
                }
            },

            // save changed name
            save: function (e) {
                var extension = this.model.get("extension"),
                    prevName;

                if (e.type === "keyup") {
                    this.keyup = true;
                } else if (e.type === "focusout" && this.keyup) {
                    this.keyup = false;
                    return;
                }

                if ((e.target.value === this.model.get("name").replace(new RegExp(extension, 'g'), "")) || (e.target.value === "")) {
                    if ($(e.target).hasClass("DialogError")) {
                        return;
                    }
                    this.render();
                } else {
                    prevName = this.model.get("path");

                    this.model.save({
                        name: e.target.value + extension
                    }, {
                        wait: true,
                        method: "renameFile",
                        prevName: prevName,
                        view: this
                    });
                }
            },

            fileAlreadyExists: function () {
                this.itemName.trigger("click");
                this.input.addClass("DialogError");

                if (this.tooltip) {
                    this.tooltip.uiTooltip('show');
                } else {
                    this.tooltip = $('<span class="Tooltip">' + tFileAlreadyExists + '</span>').uiTooltip({
                        context: this.input,
                        orientation: "right"
                    });
                }
            }
        });

    return view;
});