define(['testsuite/qunit', 'jquery', 'de_epages/order/ui/documentgenerator', '$ready!'], function (QUnit, $, de_epages) {

	var orders = [1234,5678,666];

	QUnit.module("de_epages/order/ui/documentgenerator");

	QUnit.asyncTest("de_epages/order/ui/documentgenerator", function() {

			var postData = {};

			$.ajax = function( o ){
				var jqXHR = $.Deferred();
				postData = o.data;

				jqXHR.resolve({});

				return jqXHR.promise();
			};

			postData.orders = orders;

			window.open = function (url){
				postData.printView = url;
				testcases(postData);
			};

			var generatorWidgetElem = $('<div>')
					.orderUiDocumentGenerator({
						orders: orders
					});

			generatorWidgetElem.orderUiDocumentGenerator('open');

			$("#Documents_PackingSlip").attr("checked",false);
			$("#Documents_Invoice").attr("checked",true);
			$("#PackingSlipDate").val("25/03/1989");
			$("#DeliveryDate").val("16/10/1988");
			$("#Status_ReadyForShippingOn").attr("checked",true);
			$("#Status_DispatchedOn").attr("checked",false);
			$("#Status_InvoicedOn").attr("checked",false);
			$("#ModeCreateAndPrint").attr("checked",true);

			setTimeout(function () {
				$("#Generate").trigger('click');
			}, 1000);

			setTimeout(function () {
				$("button:last").trigger('click');
			}, 1500);

			function testcases(postData) {

				QUnit.equal(
					generatorWidgetElem.orderUiDocumentGenerator('Instance') instanceof $.Widget,
					true,
					"Document generator widget is created and instance of jQuery.Widget"
				);

				$.each(['_init', '_create', '_updateOrder', '_generate', '_printView', '_validate', 'open', 'close', 'destroy'], function (i, fn) {

					QUnit.equal(
						$.isFunction($.ui.orderUiDocumentGenerator.prototype[fn]),
						true,
						'$.isFunction(' + fn + '):'
					);

				});

				QUnit.equal(
					$("#Status_ReadyForShippingOn").attr("checked"),
					$("#Check_ReadyForShippingOn").attr("checked"),
					"as expected the status ready for shipping has updated the matching checkbox at the orderpage"
				);

				QUnit.equal(
					postData.IssueDate,
					"25/03/1989",
					"the post method sends the correct issue date which has been initiated by the testscript in the dialog field"
				);

				QUnit.equal(
					postData.DeliveryDate,
					"16/10/1988",
					"the post method sends the correct date of delivery which has been initiated by the testscript in the dialog field"
				);

				QUnit.equal(
					postData.ChangeAction,
					"JSONGenerateDocuments",
					"the called ChangeAction which creates the Documents is correct"
				);

				QUnit.equal(
					postData.printView,
					"?ViewAction=MBO-ViewPrintMultipleDocuments&ObjectID=0&PopUp=1&Print=1&PrintPackingSlips=0&PrintInvoices=1&ListObjectID=1234&ListObjectID=5678&ListObjectID=666",
					"The view action to open the print view in a new window works right. the ObjectID which is the SiteId(1337) and the ListObjectID whis is the Order-ObjectId(12345) is parsed correctly"
				);

				disabledButtonTest();

				destroy();

				QUnit.start();
			}

			function disabledButtonTest(){
				generatorWidgetElem.orderUiDocumentGenerator('open');

				$(":checkbox[id^=Documents_]")
					.attr("checked",false)
					.trigger('change');

				QUnit.equal(
					$("#Generate").hasClass("Disabled"),
					true,
					"If both checkboxes for available documents are unchecked, the <Create>-Button gets disabled"
				);

				generatorWidgetElem.orderUiDocumentGenerator('close');
			}

			$("#documentGeneratorA").on('click',function(){
				generatorWidgetElem.orderUiDocumentGenerator('open');
			});

			function destroy(){
				generatorWidgetElem.orderUiDocumentGenerator('destroy')

				QUnit.equal(
					generatorWidgetElem.hasClass("ui-dialog"),
					false,
					"successfully destroyed when last div does not have the class ui-dialog anymore, so the hasClass check is false"
				);

				QUnit.equal(
					generatorWidgetElem.data("orderUiDocumentGenerator"),
					undefined,
					"Document generator widget was successfully destroyed"
				);
			}
		});
});