/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.order.ui.documentGenerator
 *
 * @revision		$Revision: 1.12 $
 */

define( "de_epages/order/ui/documentgenerator", [
	"jquery",
	"ep",
	"de_epages",
	"$dict!../dictionary",
	"$tmpl!./documentgenerator",

	"jquery/metaparse",
	"jquery/ui/widget",
	"jquery/ui/draggable",
	"jquery/i18n",
	"ep/dict",
	"ep/ui/input",
	"ep/ui/dialog",
	"ep/ajax",
	"ep/ui/datepicker"
], function ( $, ep, de_epages, presentationDict, tmplDocumentGenerator ) {

/*
 * @dictionary		de_epages.order.dictionary
 *
 * @translation		{SetStatusInvoiced}
 *					{DeliveryDate}
 *					{SetStatusDispatched}
 *					{SelectedOrders}
 *					{Invoice}
 *					{SetStatusReadyForShipping}
 *					{PackingSlip}
 *					{CreateDocuments}
 *					{Documents}
 *					{InvoicePackingSlipDate}
 *					{CreateAndPrintDocuments}
 *					{WaitForGenerateDocuments}
 */

/*
 * @dictionary		ep.dict
 *
 * @translation		{Create}
 *					{Cancel}
 *					{Action}
 */

	$.widget( 'ui.orderUiDocumentGenerator', {

		options: {
			orders    :   undefined,
			callback  :   undefined
		},

		_create: function () {
			this._createDialog();
		},

		_createDialog: function () {
			var self = this;

			$.extend(
				self,
				tmplDocumentGenerator()
					.dictParse(presentationDict, true)
					.tmplItem('elements')
			);

			self.dialog
				.find('.ep-js')
				.metaparse();

			if (this.options.orders !== undefined) {
				self.dialog
					.find('h4:first')
					.text( (presentationDict.translate('{SelectedOrders}') + ": " + this.options.orders.length) );
			}

			self.dialog
				.uiDialog({
					title: presentationDict.translate('{CreateAndPrintDocuments}'),
					modal: true,
					width: '400',
					height: 'auto',
					buttons: {
						'Generate' : {
							text: presentationDict.translate('{Create}'),
							id: 'Generate',
							click: function () {
								self._generate();
							}
						},
						'Close': {
							text: presentationDict.translate('{Cancel}'),
							click: function () {
								self.close();
							}
						}
					},
					autoOpen: false,
					create: function () {
						self._validate()
					}
				});
		},

		_updateOrder: function (readyForShipping,dispatched,invoiced) {
			readyForShipping ? $("#Check_ReadyForShippingOn").attr("checked",true) : $("#Check_ReadyForShippingOn").attr("checked",false);
			dispatched ? $("#Check_DispatchedOn").attr("checked",true) : $("#Check_DispatchedOn").attr("checked",false);
			invoiced ? $("#Check_InvoicedOn").attr("checked",true) : $("#Check_InvoicedOn").attr("checked",false);
		},

		_generate: function () {
			var self = this;

			if (!$("#Generate").hasClass("Disabled")) {
				var create = this.create.attr("checked")?true:false,
					createandprint = this.createAndPrint.attr("checked")?true:false,
					createPackingSlip = this.CreatePackingSlip.attr("checked")?true:false,
					createInvoice = this.CreateInvoice.attr("checked")?true:false,
					packingSlipDate = this.PackingSlipDate.val(),
					deliveryDate = this.DeliveryDate.val(),
					readyForShipping = this.ReadyForShipping.attr("checked")?true:false,
					dispatched = this.Dispatched.attr("checked")?true:false,
					invoiced = this.Invoiced.attr("checked")?true:false,
					objectId = ep.config.objectId;

				ep.ajax({
					dataType: "json",
					// use a synchronous request to avoid pop up blockers when using window.open
					async: false,
					data: {
						'ObjectID' : ep.config.siteId,
						'OrderList': (self.options.orders) ? self.options.orders : [objectId],
						'ChangeAction' : 'JSONGenerateDocuments',
						'Locale' : ep.config.locale,
						'DeliveryDate' : deliveryDate,
						'IssueDate' : packingSlipDate,
						'createInvoice' : createInvoice,
						'createPackingSlip' : createPackingSlip,
						'ReadyForShippingOn' : readyForShipping,
						'DispatchedOn' : dispatched,
						'InvoicedOn' : invoiced
					},
					cache: false,
					type: "post",
					traditional: true
				})
				.done(function (data) {
					if(typeof self.options.callback == "function") {
						self.options.callback(data);
					}

					self._updateOrder(readyForShipping, dispatched, invoiced);
					self._printView();

					// Destroy dialog
					self.dialog.remove();
					self.dialog = null;
				});
			}
		},

		_printView: function () {
			var printString,
				printPackingSlips = $("#Documents_PackingSlip").attr("checked") ? 1 : 0,
				printInvoices = $("#Documents_Invoice").attr("checked") ? 1 : 0,
				partOfString = "&PopUp=1&Print=1&PrintPackingSlips="+printPackingSlips+"&PrintInvoices="+printInvoices;
			if(this.options.orders !== undefined && ( this.options.orders.length > 1 || ep.config.siteId == ep.config.objectId ) ) {
				var list = this.options.orders,
					printString="?ViewAction=MBO-ViewPrintMultipleDocuments&ObjectID="+ep.config.objectId+partOfString;
				$.each(list,function (index, value) {
					printString = printString + '&'+'ListObjectID=' + value;
				});
			}
			else{
				printString="?ViewAction=MBO-ViewPrintMultipleDocuments&ObjectID="+ep.config.siteId+partOfString+"&ListObjectID="+ep.config.objectId;
			}
			$("#DocumentGeneratorMessage")
				.removeClass("HideElement")
				.find("a")
				.attr("href", printString);

			if (this.createAndPrint.attr("checked")) {
				window.open(printString);
			}
		},

		_validate: function () {
			 var self = this;
			 $.each([self.CreatePackingSlip, self.CreateInvoice], function (index, value) {
				this.on('change', function () {
					$("#Generate")[(!self.CreatePackingSlip.is(":checked") && !self.CreateInvoice.is(":checked"))? 'addClass' : 'removeClass']("Disabled");
				});
			});
			$.each([self.PackingSlipDate, self.DeliveryDate], function (index, value) {
				this.on('change', function () {
					$("#Generate")[(self.PackingSlipDate.val()=="" || self.DeliveryDate.val()=="")? 'addClass' : 'removeClass']("Disabled");
				});

			});
		},

		open: function () {
			var self = this;

			if (self.dialog) {
				self.dialog.uiDialog('open');
			}
			else {
				self._createDialog();
			}

		},

		close: function () {
			var self = this;

			if (self.dialog) {
				self.dialog.uiDialog('close');
			}

		},

		_init: function () {
			var self = this,
				epDate = new ep.Date();

			$.each([self.PackingSlipDate, self.DeliveryDate], function (index, value) {
				this.val(epDate.getFormat('d'));
			});
		},

		destroy: function () {
			this.dialog.remove();

			this._superApply(arguments);
		}
	});

	return de_epages;

});