define(['testsuite/qunit', 'jquery', 'de_epages/presentation/ajaxtransport', 'testsuite/sinon', 'testsuite/fake', '$ready!'], function (QUnit, $, de_epages, sinon, FAKE) {

	QUnit.module("de_epages/presentation/ajaxtransport");

	QUnit.asyncTest("de_epages/presentation/ajaxtransport", function() {

		// define responses (sandbox.server.responses)
		var sandbox = sinon.sandbox.create(),
			server = sandbox.useFakeServer();

		server.autoRespond = true;
		server.autoRespondAfter = 0;
		server.respondWith("POST", "1?ErrorAction=JSONAjaxTransportFileError", [404, {}, '']);
		server.respondWith("POST", "0?ErrorAction=JSONAjaxTransportFileError", [200, {"Content-Type": "application/json"}, '{"upload":"Ok"}']);

		// create faked file inputs
		var inputs = [
				FAKE('HTMLInputElement',{
					type: 'file',
					size: 20,
					files: [
						FAKE('File', 'example-0.html', 'text/html', 512 ),
						FAKE('File', 'example-0.jpg', 'image/jpeg', 2048 )
					]
				}),
				FAKE('HTMLInputElement',{
					type: 'file',
					size: 20,
					files: [
						FAKE('File', 'example-1.jpg', 'image/jpeg', 2048 )
					]
				}),
				FAKE('HTMLInputElement',{
					type: 'file',
					size: 20,
					files: [
						FAKE('File', 'example-2.jpg', 'image/jpeg', 40960 )
					]
				}),
				FAKE('HTMLInputElement',{
					type: 'file',
					size: 20,
					files: [
						FAKE('File', 'example-3.json', 'application/json', 256 )
					]
				}),
				FAKE('HTMLInputElement',{
					type: 'file',
					size: 20,
					files: [
						FAKE('File', 'example-4.jpg', 'image/jpeg', 2048 )
					]
				})
			],
			i = 0;

		// test vars
		var t = [
				{},
				{},
				{},
				{},
				{}
			];

		$(':file')
			.change(function( event ){
				var j = i + 0,
					ajax = $.ajax({
						file:		inputs[ j ],
						url:		j.toString(),
						fileSizeMax:	4096,
						fileMimeAccept: 'text/*,image/*',
						dataType:	'json',
						progress:	function( event, statusText, jqXHR ){
						//	console.log( event, statusText, jqXHR, 'DATA:', event.data );
							t[ j ].progress = $.extend({},event.data);
						}
					})
					.done(function( data, statusText, jqXHR ){
					//	console.log( j, data, statusText, jqXHR );
						t[ j ].success = $.merge([],data);
					})
					.fail(function( jqXHR, statusText, errorThrown ){
					//	console.log( j, jqXHR, statusText, errorThrown );
						t[ j ].error = errorThrown;
					});

				if( i >=4 ){
					ajax.abort();
				}

				i++;
			})
			.trigger('change')
			.trigger('change')
			.trigger('change')
			.trigger('change')
			.trigger('change');


		setTimeout(function(){
			var k = 0;

			if(!$.browser.msie){
				QUnit.deepEqual(
					t[k].progress.item,
					{"name":"NewFile","fileName":"example-0.jpg","fileSize":2048,"fileType":"image/jpeg"},
					"$.ajax( case["+k+"] ).progress()"
				);
			}

			if(!$.browser.msie){
				QUnit.deepEqual(
					t[k].success,
					[{"upload":"Ok"},{"upload":"Ok"}],
					"$.ajax( case["+k+"] ).success()"
				);
			}

			if(!$.browser.msie){
				QUnit.equal(
					t[k].error,
					undefined,
					"$.ajax( case["+k+"] ).fail()"
				);
			}

			k++;

			if(!$.browser.msie){
				QUnit.deepEqual(
					t[k].progress.item,
					{"name":"NewFile","fileName":"example-1.jpg","fileSize":2048,"fileType":"image/jpeg"},
					"$.ajax( case["+k+"] ).progress()"
				);
			}

			QUnit.equal(
				t[k].success,
				undefined,
				"$.ajax( case["+k+"] ).success()"
			);

			if(!$.browser.msie){
				QUnit.equal(
					t[k].error,
					'Items of Request Failed',
					"$.ajax( case["+k+"] ).fail()"
				);
			}


			k++;

			if(!$.browser.msie){
				QUnit.deepEqual(
					t[k].progress.item,
					{"name":"NewFile","fileName":"example-2.jpg","fileSize":40960,"fileType":"image/jpeg"},
					"$.ajax( case["+k+"] ).progress()"
				);
			}


			QUnit.equal(
				t[k].success,
				undefined,
				"$.ajax( case["+k+"] ).success()"
			);


			if(!$.browser.msie){
				QUnit.equal(
					t[k].error,
					'Items of Request Failed',
					"$.ajax( case["+k+"] ).fail()"
				);
			}


			k++;

			if(!$.browser.msie){
				QUnit.deepEqual(
					t[k].progress.item,
					{"name":"NewFile","fileName":"example-3.json","fileSize":256,"fileType":"application/json"},
					"$.ajax( case["+k+"] ).progress()"
				);
			}

			QUnit.equal(
				t[k].success,
				undefined,
				"$.ajax( case["+k+"] ).success()"
			);

			if(!$.browser.msie){
				QUnit.equal(
					t[k].error,
					'Items of Request Failed',
					"$.ajax( case["+k+"] ).fail()"
				);
			}

			k++;

			if(!$.browser.msie){
				QUnit.equal(
					t[k].progress,
					undefined,
					"$.ajax( case["+k+"] ).progress()"
				);
			}

			QUnit.equal(
				t[k].success,
				undefined,
				"$.ajax( case["+k+"] ).success()"
			);

			if(!$.browser.msie){
				QUnit.equal(
					t[k].error,
					'abort',
					"$.ajax( case["+k+"] ).abort()"
				);
			}

			QUnit.start();

		}, 500);
	});
});