define(['testsuite/qunit', 'jquery', 'de_epages/product/ui/appointmentpicker', 'ep/date', '$ready!'], function (QUnit, $, de_epages, ep) {

	QUnit.module("de_epages/product/ui/appointmentpicker");

	QUnit.asyncTest("de_epages/product/ui/appointmentpicker", function() {

    	var RunError = false;

//==================================================================================//
// FAKE: Requests
//==================================================================================//

    	var DayListCount = 0,
    		DayListMap = [
    			[1,0,0,1,0,0,2,2,2,2,0,0,0,0,1,2,2,2,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,0,2,2,2],
    			[0,0,0,0,0,0,0,0,0,0,1,2,2,2,2,2,1,1,2,1,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0],
    			[1,1,1,1,0,0,0,1,0,0,1,0,0,1,1,2,1,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1],
    			[0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,0,2,2,2,2,2,0,0,0,0,0,0,0,0,0,0,0]
    		];

    	// define fake requests
    	$.ajax = function( options ){
    		var jqXHR	= $.Deferred(),
    			data	= options.data,
				result	= {};

			// date period loop setup
			var i = new ep.Date().setFormat(data.StartDate, "S"),
				l = new ep.Date().setFormat(data.EndDate  , "S").getTime(),
				temp;

			while( i.getTime() < l ){
				temp = {
					weekday	: i.getDay(),
					add		: {},
					j		: i.clone().setHours(8),
					m		: i.clone().setHours(20).getTime()
				};

				if( temp.weekday != 0 && temp.weekday!=6  ){
					temp.add["IsAvailable"] = 1;
				}
				else{
					temp.add["IsAvailable"] = 0;
				}
				if( !data.IsAvailableOnly ){
					temp.add["DayList"] = [];

					if( (temp.DayListMap = DayListMap[ DayListCount ] || []).length ){
						temp.DayListCount = 0;

						while( temp.j.getTime() < temp.m ){
							temp.add["DayList"].push(
								[
									temp.j.getFormat("S"),
									temp.DayListMap[ temp.DayListCount ] || 0
								]
							);

							// increment DayList times
							temp.j.addMinutes(10);
							temp.DayListCount++;
						}
					}

					// increment Map of DayList settings
					DayListCount++;
				}
				// add day to result
				result[ i.getFormat("S") ] = temp.add;
				// increment Days
				i.addDate(1);
			}

			// trigger ajax success / error
			if( RunError ){
				jqXHR.reject(jqXHR,'error',"Error message");
				RunError = false;
			}
			else{
				jqXHR.resolve(result,'success',jqXHR);
			}

    		return jqXHR;
    	};

//==================================================================================//
// BEGIN: TEST
//==================================================================================//

    	// init appointmentpicker
    	var input = $('input[name=date]')
    			.productUiAppointmentpicker({
					currentDate: {
						year:  2012,
						month: 7,
						day:   12
					},
					appointment: {
						id					: 0,
						quantityMin			: 1,
						quantityRequested	: 1,
						quantityAvailable	: 2,
						minutesRequested	: 20
					}
				}),
    		button = input
    			.next()
    			.trigger('click'),
    		picker = $('.ep-uiDatepicker-container'),
    		tstamp = $('.ep-uiDatepicker-daySelected a').data('tstamp');

		// is defined
		QUnit.equal(
			$.isFunction($.ui.productUiAppointmentpicker),
			true,
			"$.isFunction($.ui.productUiAppointmentpicker)"
		);

		// prototype methods defined
		$.each( ['_create','_createTimeSelect','_createCalendars','_calendarData','_createTimeSelectData','_setTimeVal','_getTimeVal','_loadDayList','updateAppointment'], function( i, fn ){

			QUnit.equal(
		 	   $.isFunction( $.ui.productUiAppointmentpicker.prototype[fn] ),
		 	   true,
		 	   "$.isFunction( $.ui.productUiAppointmentpicker.prototype['"+  fn +"'] )"
			);

		});

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			57,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-content tbody tr:eq(1) a').length"
		);

		// go to next month and select an other day
		$('.ep-uiDatepicker-next button').trigger('click');
		$('.ep-uiDatepicker-content a').eq(10).trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			60,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		// change quantity
		input.productUiAppointmentpicker('updateAppointment',{
				quantityRequested	: 2
			});

		// go to previos month and select the first selected day again
		$('.ep-uiDatepicker-prev button').trigger('click');
		$('.ep-uiDatepicker-content a[data-tstamp="'+tstamp+'"]').trigger('click');

		// depending on the chosen day the available options can vary -> therefore two length' are possible
		var varyLen = (picker.find('.ep-uiDatepicker-timeBar select option').length == 44) ? 44 : 38;

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			varyLen,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-content tbody tr:eq(1) a').length"
		);

		// select an other day
		$('.ep-uiDatepicker-next button').trigger('click');
		$('.ep-uiDatepicker-content a').eq(9).trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			41,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-content tbody tr:eq(1) a').length"
		);

		// click today button
		$('.ep-uiDatepicker-today button').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			38,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-content tbody tr:eq(1) a').length"
		);

		// select a time
		picker.find('.ep-uiDatepicker-timeBar select option').eq(26).attr('selected',true);
		picker.find('.ep-uiDatepicker-timeBar select').trigger('change');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option:selected').text(),
			'16:00',
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option:selected').text()"
		);

		// select an other day and simulate a server error
		RunError = true;
		$('.ep-uiDatepicker-content a').eq(9).trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			1,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-content tbody tr:eq(1) a').length"
		);

		var clone = picker.find('.ep-uiDatepicker-timeBar select option').clone(true);

		QUnit.equal(
			clone.val(),
			"Lädt...",
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select').val()"
		);

		QUnit.equal(
			$('body > .ui-dialog').length,
			1,
			"$('body > .ui-dialog').length"
		);

		// close error message and open picker again
		$('.ui-dialog-titlebar-close').trigger('click');
		button.trigger('click');

		// go to next next next month and simulaye server error
		$('.ep-uiDatepicker-next button').trigger('click').trigger('click');
		RunError = true;
		$('.ep-uiDatepicker-next button').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			1,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select option').length"
		);

		$('.ep-uiDatepicker-prev button').trigger('click');
		$('.ep-uiDatepicker-next button').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-content tbody tr:eq(1) a').length"
		);

		clone = picker.find('.ep-uiDatepicker-timeBar select option').clone(true);

		QUnit.equal(
			clone.val(),
			"Ausgebucht",
			"$('.ep-uiDatepicker-container').find('.ep-uiDatepicker-timeBar select').val()"
		);

		// close error message and open picker again
		$('.ui-dialog-titlebar-close').trigger('click');
		button.trigger('click');

		// click done button
		$('.ep-uiDatepicker-done button').trigger('click');

		QUnit.start();
    });

});