define(['testsuite/qunit', 'jquery', 'de_epages/seo/ui/seocheckwidget', 'testsuite/sinon', 'testsuite/sinon/server', 'testsuite/sinon/ie', '$ready!'], function (QUnit, $, de_epages, sinon) {

	QUnit.module("de_epages/seo/ui/seocheckwidget");

	QUnit.asyncTest("de_epages/seo/ui/seocheckwidget", function () {

		// is defined
		QUnit.equal(
			$.isFunction($.ui.seoUiSeoCheckWidget),
			true,
			"$.isFunction($.ui.seoUiSeoCheckWidget)"
		);

		// prototype methods defined
		var fnCheck = true;
		$.each(['_create', '_init', '_getDialogContent', '_buildTitleGauge', '_refreshTitleGauge'], function (i, fn) {
			fnCheck = !$.isFunction($.ui.seoUiSeoCheckWidget.prototype[fn]) ? false : fnCheck;
		});

		QUnit.equal(
			fnCheck,
			true,
			"each => $.ui.seoUiSeoCheckWidget.prototype => $.isFunction()"
		);

		// response for SEOQuality
		var xmlResponse = ([
				'<div class="seoCheck" data-newseoquality="15"><ul class="seoTest"><li class="seoInfoBox red"> ',
				'  <h4>Use short URL</h4> ',
				'  <span class="seoInfoFlag">✘</span> ',
				'  <div class="description">Description</div> ',
				'</li><li class="seoInfoBox red"> ',
				'  <h4>Optimise "Long description" text</h4> ',
				'  <span class="seoInfoFlag">✘</span> ',
				'  <div class="description">Description</div> ',
				'</li><li class="seoInfoBox yellow"> ',
				'  <h4>Optimise "Description" text</h4> ',
				'  <span class="seoInfoFlag">!</span> ',
				'  <div class="description"> ',
				'     The description is too short. (84 Characters) Between 90 and 140 characters are recommended. ',
				'    <div style="margin:10px;padding:10px;border:1px solid #000;"> ',
				'      <strong>Description:</strong> ',
				'      <br> ',
				'      <em>A backpack is your most important companion and keeps your hands free for adventure.</em> ',
				'      <br> ',
				'      <span style="font-size:smaller">Words: <strong>14</strong>, Characters: <strong>84</strong></span> ',
				'    </div> ',
				'  </div> ',
				'</li> ',
				'<li class="seoInfoBox green"> ',
				'  <h4>Meaningful page title</h4> ',
				'  <span class="seoInfoFlag">✔</span> ',
				'  <div class="description">Description</div> ',
				'  <div style="margin:10px;padding:10px;border:1px solid #000;"> ',
				'    <strong>Page title:</strong> ',
				'    <br> ',
				'    <em>Backpacks - Milestones</em> ',
				'    <br> ',
				'    <span style="font-size:smaller">Words: <strong>3</strong>, Characters: <strong>22</strong></span> ',
				'  </div> ',
				'</li> ',
				'<li class="seoInfoBox green"> ',
				'  <h4>Add image</h4> ',
				'  <span class="seoInfoFlag">✔</span> ',
				'  <div class="description"> ',
				'     An image is added to the page. ',
				'  </div> ',
				'</li> ',
				'<li class="seoInfoBox green"> ',
				'  <h4>Expedient category depth</h4> ',
				'  <span class="seoInfoFlag">✔</span> ',
				'  <span>This page is located <b>1</b> level(s) below the start page.</span> ',
				'</li> ',
				'</ul></div>'
			])
			.join('\n');

		var sandbox = sinon.sandbox.create();
		var server = sandbox.useFakeServer();

		// define responses (sandbox.server.responses)
		server.respondWith(
			"GET",
			"",
			[
				200,
				{
					"Content-Type": "text/html"
				},
				xmlResponse
			]
		);

		server.respondWith(
			"POST",
			"",
			[
				200,
				{
					"Content-Type": "application/json"
				},
				'{}'
			]
		);

		var dialog;

		if (!dialog) {
			dialog = $('<div>')
				.uiDialog({
				autoOpen: false,
				title: '<span class="ep-SEOGaugeWidgetTitle">{SEOCheck}</span>',
				buttons: {
					"Close": {
						text: '{Close}',
						click: function () {
							ep(this)
								.uiDialog("close");
						}
					}
				}
			});
		}

		// same SEOQuality
		dialog.seoUiSeoCheckWidget({
			'currentSEOQuality': 15
		});

		// respond request by fakeServer
		server.respond();

		// define responses for the requests
		sandbox.server.requests[0].respond(sandbox.server.responses[0]);

		// respond second request only if necessary (only send if currentSEOQuality values are different)
		if (sandbox.server.requests[1] !== undefined) {
			sandbox.server.requests[1].respond(sandbox.server.responses[1]);
		}

		QUnit.ok($(dialog).seoUiSeoCheckWidget('option', 'currentSEOQuality') == 15, 'same currentSEOQuality');

		// different SEOQuality
		dialog.seoUiSeoCheckWidget({
			'currentSEOQuality': 16
		});

		// respond request by fakeServer
		server.respond();

		// define responses for the requests
		sandbox.server.requests[0].respond(sandbox.server.responses[0]);

		// respond second request only if necessary (only send if currentSEOQuality values are different)
		if (sandbox.server.requests[1] !== undefined) {
			sandbox.server.requests[1].respond(sandbox.server.responses[1]);
		}

		QUnit.ok($(dialog).seoUiSeoCheckWidget('option', 'currentSEOQuality') != 15, 'different currentSEOQuality');

		QUnit.start();

	});
});