/*globals CKEDITOR, window, jq, jQuery*/

(function($){
	'use strict';

	var linkpicker,
		currentEditor,
		typeName = 'linkpicker';

	CKEDITOR.plugins.add( 'eplink',{
		init: function( editor ){

			// if the context menu plugin is loaded, register the menu items
			if(editor.addMenuItems){
				editor.addMenuItems({
					eplink: {
						label : editor.lang.link.menu,
						command : 'eplink',
						group : 'link',
						order : 1
					}
				});

				// remove not used link entries from context menue
				editor.removeMenuItem('link');
			}

			// if the "contextmenu" plugin is loaded, register the listeners.
			if(editor.contextMenu){
				editor.contextMenu.addListener(function(element, selection){
					if( !element || element.isReadOnly()){
						return null;
					}

					var anchor = CKEDITOR.plugins.link.tryRestoreFakeAnchor(editor, element) || CKEDITOR.plugins.link.getSelectedLink( editor ),
						menu = {};

					if ( !anchor ) {
						return null;
					}

					if(anchor.getAttribute('href') && anchor.getChildCount()){
						menu = {eplink : CKEDITOR.TRISTATE_OFF};
					}

					return menu;
				});
			}

			editor.addCommand( 'eplink',{
				exec : function( ExecEditor ){
					currentEditor = ExecEditor;
					var selection = ExecEditor.getSelection(),
						selectedText = selection.getSelectedText(),
						selectedLink = CKEDITOR.plugins.link.getSelectedLink(ExecEditor),
						selectedElem = selection.getSelectedElement(),
						anchorData = currentEditor.getData(),
						anchorArray = [],
						frame = window,
						selectedUrl,
						imageSelected;

					if(selectedLink){
						selectedLink = $(selectedLink.$);
						selectedUrl = selectedLink.attr('href');

						selectedText = selectedLink.text();

						if(selectedLink.children('img').length === 1){
							imageSelected = true;
						}

						// select link element
						ExecEditor.getSelection().selectElement( CKEDITOR.plugins.link.getSelectedLink(ExecEditor) );
					}
					else if(selectedElem && selectedElem.$.tagName.toLowerCase() === "img"){
						imageSelected = true;
					}

					

					if(anchorData){
						$(anchorData).find('a[name]').each(function(){
							anchorArray.push($(this).attr('name'));
						});
					}

					// check in which context we're requiring the linkpicker (are we inside an iframe?)
					if (window.top !== window.self) {
						try {
							frame = window.top;
						}
						catch (error) {}
					}

					// require linkpicker
					frame.require(['de_epages/presentation/linkpicker'
					], function(Linkpicker) {

						// initialize with default options
						linkpicker = new Linkpicker({
							use_absolute: true, //currentEditor.config.useAbsolute ? true : false,
							use_ssl: false,
							callback: function(title, url, newFlag, linkType) {
								if(imageSelected){
									$(selectedElem.$).attr({alt: title, title: title});
								}
								var linkContent = '<a data-type="'+typeName+'" href="' + url + '"' + (newFlag ? ' target="_blank"' : '') + ' data-link-type="' + linkType + '">' + (imageSelected ? selectedElem.$.outerHTML : title) + '</a>';
								currentEditor.insertHtml(linkContent);
							}
						});

						// opening linkpicker and setting options
						linkpicker.open({
							linkText: selectedText,
							linkUrl: selectedUrl || '',
							is_linkTargetBlank: $(editor.getSelection().getStartElement().$).attr('target') === '_blank' ? true : false,
							linkType: $(editor.getSelection().getStartElement().$).data('linkType')
						});
					});
				}
			});

			editor.ui.addButton( 'eplink',{
				label: editor.lang.link.toolbar,
				command: 'eplink',
				className: 'cke_button_link'
			});

			editor.on( 'doubleclick', function( evt ){
		
				// if sender of double click event was a linkpicker link -> open necessary dialog
				if(jq(editor.getSelection().getStartElement().$).data('type') === typeName ){
					editor.execCommand('eplink');
					delete evt.data.dialog;
				}
			});

		}
	});
}(jQuery));