/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages/shopmailtype/ui/editmailaddress
 *
 * @revision		$Revision: 1.8 $
 */

define("de_epages/shopmailtype/ui/editmailaddress", [
	"jquery",
	"ep",
	"de_epages",
	"$tmpl!./editmailaddress",

	"jquery/ui/widget",
	"ep/ajax",
	"ep/ui/simpledialog",
	"ep/ui/validate",
	"ep/ui/input"
], function ($, ep, de_epages, tmplEditMailadress) {

	// Widget.
	$.widget('ui.shopmailtypeUiEditmailaddress', {

		options : {
			wording : {},
			data : {'dummy' : []},
			name : 'dummy',
			multiple : true,
			showSetDefault : false
		},

		_create : function () {
			var self = this,
					o = self.options;

			// Merge in options from data attribute.
			$.extend(true, o, $.parseJSON(self.element.attr('data-de_epages-shopmailtype-ui-editmailaddress')));

			// Set corresponding input element.
			if (!o.name){
				return self.destroy();
			}

			self.input = $('input[name="' + o.name + '"]');

			// Render template.
			self.content = $('<form>').append(tmplEditMailadress(o.data[o.name].length ? o.data[o.name] : [{}] , {wording : o.wording}));
			// Init delete event listeners.
			self.content.on('click', '.ico_s_delete', $.proxy(self, '_deleteMail'));

			// Init *uiValidate* and *uiInput*.
			$(self.content.find('input[type="email"]')).uiValidate({type : 'email'});
			$(self.content.find('input[type="text"]')).uiInput();

			// Add footer.
			// i.e. *setDefault* checkbox or *addMail* button.
			self.footer = $('<footer>');
			if (o.multiple) {
				$('<span  style="padding-left:158px;" class="DisplayBlock PseudoLink CursorPointer">')
					.text(o.wording.addMail)
					.on('click', $.proxy(self, self._addMail))
					.appendTo(self.footer);
			}
			else {
				if (o.showSetDefault) {
					// Init and render template.
					!$.template['de_epages.shopmailtype.inc.setdefault'] &&	$.template('de_epages.shopmailtype.inc.setdefault','<div class="InputRow">' +
						'<input type="checkbox" name="setdefault" id="setdefault"/>' +
						'<label class="InputLabellingXWide AlignLeft" for="setdefault">${$item.wording.setDefault}</label>' +
						'</div>');

					self.setDefault = tmplEditMailadress({}, {wording : o.wording})
						.appendTo(self.footer)
						.find('input[type="checkbox"]')
						.uiInput();
				}
			}

			// Init *uiSimpleDialog*.
			self.element = $(self.element).uiSimpledialog({
				content : [self.content[0], self.footer[0]],
				buttons : [{
					text : o.wording.apply,
					callback : function () {
						if (self.content.find('input[type="email"]:invalid').length) {
							$(self.content.find('input[type="email"]:invalid')).trigger('validate.uiValidate');
							return;
						}
						else {
							self._apply();
							self._close();
						}
					}
				}]
			});
		},

		_deleteMail : function (event) {
				event.stopPropagation();
				// Otherwise *self.dialog._close()* is called, since
				// the click was fired by an element which at that point
				// is no longer part of the dialog.
				$(event.target).parent().next().remove();
				$(event.target).parent().remove();
		},

		_addMail : function () {
			var self = this,
					o = self.options,
					inputrow = tmplEditMailadress({}, {wording : o.wording});

			// Init *uiValidate* and *uiInput*.
			$(inputrow.find('input[type="email"]')).uiValidate({type : 'email'});
			$(inputrow.find('input[type="text"]')).uiInput();

			inputrow.appendTo(self.content);
		},

		_apply : function () {
			var self = this,
					o = self.options,
					value = '',
					inputs = self.content.find('input[type="text"], input[type="email"]'),
					names = inputs.filter('[type="text"]'),
					name = '',
					doajax =  self.setDefault && self.setDefault.is(':checked');

			// Let us add all those emails and names together.
			inputs.filter('[type="email"]').each(function (index, email){
				email = email.value;
				name = names[index].value;
				// Set default Sender Address, if the checkbox is checked.
				if ((index === 0) && doajax){
				  ep.ajax({
					type : 'POST',
					dataType : 'json',
					data : {
						'DefaultEmail' : email,
						'DefaultName' : name,
						'ViewAction' : 'JSONViewResponse',
						'ChangeAction' : 'JSONMakeDefaultSender',
						'ObjectID' : ep.config.objectId
					}
				});}
				value = value + (value ? ';' : '') + name + (name ? ' <' : '') + email + (name ? '>' : '');
			});

			// Output email string into input.
			self.input.val(value);
		},

		_close : function () {
			this.element.uiSimpledialog('close');
		},

		destroy : function () {
			var self = this;
			self.content && self.content.remove();
			self.element.uiSimpledialog('destroy');
			self._superApply(arguments);
		}

	});

	return de_epages;

});